import { logWarn, yellow, registerPlugin } from '@scullyio/scully';
import highlight from 'highlight.js';
import { __awaiter, __generator, __values, __read, __spread } from 'tslib';
import { readFileSync } from 'fs';
import { JSDOM } from 'jsdom';
import { parse } from 'marked';

var scullyBegin = '<!--scullyContent-begin-->';
var scullyEnd = '<!--scullyContent-end-->';
function renderPlugin(html, route) {
    return __awaiter(this, void 0, void 0, function () {
        var sectionBuilders, attr, additionalHTML, sectionBuilders_1, sectionBuilders_1_1, builder, markdownHtmls, _a, _b, markdownFileSource, originalFile, options, markdownHtml, htmlWithNgAttr;
        var e_1, _c, e_2, _d;
        return __generator(this, function (_e) {
            sectionBuilders = route.config.sectionBuilders;
            try {
                attr = '';
                try {
                    attr = getIdAttrName(html.split('<scully-content')[1].split('>')[0].trim());
                }
                catch (e) {
                    logWarn("\n        ----------------\n        Error, missing \"" + yellow('<scully-content>') + "\" in route \"" + yellow(route.route) + "\"\n        without <scully-content> we can not render this route.\n        Make sure it is in there, and not inside any conditionals (*ngIf)\n        You can check this by opening \"" + yellow("://localhost:4200/" + route.route) + "\"\n        when you serve your app with " + yellow('ng serve') + " and then in the browsers console run:\n        " + yellow("document.querySelector('scully-content')") + "\n        ----------------\n        ");
                }
                additionalHTML = '';
                try {
                    for (sectionBuilders_1 = __values(sectionBuilders), sectionBuilders_1_1 = sectionBuilders_1.next(); !sectionBuilders_1_1.done; sectionBuilders_1_1 = sectionBuilders_1.next()) {
                        builder = sectionBuilders_1_1.value;
                        markdownHtmls = [];
                        try {
                            for (_a = (e_2 = void 0, __values(builder.markdownFileSources)), _b = _a.next(); !_b.done; _b = _a.next()) {
                                markdownFileSource = _b.value;
                                originalFile = readFileSync(markdownFileSource, 'utf-8');
                                options = {
                                    highlight: function (code, language) {
                                        var hljs = highlight;
                                        var validLanguage = hljs.getLanguage(language)
                                            ? language
                                            : 'plaintext';
                                        return hljs.highlight(validLanguage, code).value;
                                    },
                                    pedantic: false,
                                    gfm: true,
                                    breaks: false,
                                    sanitize: false,
                                    smartLists: true,
                                    smartypants: false,
                                    xhtml: false
                                };
                                markdownHtml = parse(originalFile, options);
                                markdownHtmls.push(markdownHtml);
                            }
                        }
                        catch (e_2_1) { e_2 = { error: e_2_1 }; }
                        finally {
                            try {
                                if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                            }
                            finally { if (e_2) throw e_2.error; }
                        }
                        additionalHTML = additionalHTML + builder.sectionBuilder(markdownHtmls);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (sectionBuilders_1_1 && !sectionBuilders_1_1.done && (_c = sectionBuilders_1.return)) _c.call(sectionBuilders_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                htmlWithNgAttr = addNgIdAttribute(additionalHTML, attr);
                return [2 /*return*/, insertContent(scullyBegin, scullyEnd, html, htmlWithNgAttr, getScript(attr))];
            }
            catch (e) {
                logWarn("Error, while rendering content for \"" + yellow(route.route) + "\" from files: \"" + yellow(sectionBuilders.map(function (builder) { return builder.markdownFileSources; })) + "\"");
                console.error(e);
            }
            return [2 /*return*/];
        });
    });
}
function addNgIdAttribute(html, id) {
    if (!id) {
        return html;
    }
    try {
        var dom = new JSDOM(html, { runScripts: 'outside-only' });
        var window_1 = dom.window;
        var document_1 = window_1.document;
        var walk = document_1.createTreeWalker(document_1.body);
        var cur = walk.currentNode;
        while (cur) {
            if (cur.nodeType === 1) {
                cur.setAttribute(id, '');
            }
            cur = walk.nextNode();
        }
        return document_1.body.innerHTML;
    }
    catch (e) {
        console.error(e);
    }
    return '';
}
function getIdAttrName(attrs) {
    try {
        return (attrs &&
            attrs
                .split(' ')
                .find(function (at) { return at.trim().startsWith('_ngcontent'); })
                .split('=')[0]);
    }
    catch (_a) {
        return '6';
    }
}
function getScript(attr) {
    // tslint:disable-next-line:no-unused-expression
    return "<script>try {window['scullyContent'] = {cssId:\"" + attr + "\",html:document.body.innerHTML.split('<!--scullyContent-begin-->')[1].split('<!--scullyContent-end-->')[0]};} catch(e) {console.error('scully could not parse content');}</script>";
}
function insertContent(startTag, endTag, html, insertText) {
    var extras = [];
    for (var _i = 4; _i < arguments.length; _i++) {
        extras[_i - 4] = arguments[_i];
    }
    try {
        var _a = __read(html.split(startTag), 2), openingText = _a[0], rest = _a[1];
        var _b = __read(rest.split(endTag), 2), takeout = _b[0], endText = _b[1];
        return __spread([openingText, startTag, insertText, endTag], extras, [endText]).join('');
    }
    catch (e) { }
    /** warning is already handled, only put in stub content. */
    // logWarn(`missing "${yellow('<scully-content>')}"`);
    return "<h1>Scully could not find the &lt.scully-content&gt. tag in this page.</h1>\n  <p>This error can happen when you forgot to put the  mandatory \"scully-content\" in the component that is rendering this page?</p>\n  <p>Or when the tag is not shown on page load. Did you put it inside an `*ngIf`?</p>\n  ";
}

var MultiMarkdownPageBuilderPluginName = 'MultiMarkdownPageBuilderPlugin';

function routerPlugin(route, config) {
    var additionalPostRenderers = config.postRenderers ? config.postRenderers : [];
    var routes = [
        {
            type: MultiMarkdownPageBuilderPluginName,
            route: route === '' ? '/' : route,
            config: config,
            postRenderers: __spread([MultiMarkdownPageBuilderPluginName], additionalPostRenderers),
            title: config.title
        }
    ];
    return Promise.resolve(routes);
}

var renderValidator = function (config) { return __awaiter(void 0, void 0, void 0, function () { return __generator(this, function (_a) {
    return [2 /*return*/, []];
}); }); };

var routerValidator = function (config) { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        if (config.sectionBuilders === undefined || !Array.isArray(config.sectionBuilders)) {
            return [2 /*return*/, ["Missing a sectionBuilders array in the " + MultiMarkdownPageBuilderPluginName + " config."]];
        }
        else {
            return [2 /*return*/, config.sectionBuilders.reduce(function (errors, sectionBuilder) {
                    if (sectionBuilder.sectionBuilder === undefined || !(sectionBuilder.sectionBuilder instanceof Function)) {
                        errors.push("Missing sectionBuilder function type in section builder: " + sectionBuilder);
                    }
                    if (sectionBuilder.markdownFileSources === undefined || !Array.isArray(sectionBuilder.markdownFileSources)) {
                        errors.push("Missing markdownFileSources array type in section builder: " + sectionBuilder);
                    }
                    return errors;
                }, [])];
        }
        return [2 /*return*/];
    });
}); };

var MultiMarkdownPageBuilderPlugin = MultiMarkdownPageBuilderPluginName;
registerPlugin('render', MultiMarkdownPageBuilderPlugin, renderPlugin, renderValidator);
registerPlugin('router', MultiMarkdownPageBuilderPlugin, routerPlugin, routerValidator);

/**
 * Creates a section of a page from a single markdown file, and adds the passed css selectors for inserting layout customization.
 *
 * Example markdown source file:
 * ```md
 * # Markdown Title Content
 * ```
 * Example usage:
 * ```typescript
 * export const config: ScullyConfig = {
 *   // ...
 *   routes: {
 *     '/path': {
 *       type: MultiMarkdownPageBuilderPlugin,
 *       sectionBuilders: [
 *         createSingleMarkdownSection('./homepage/installation.md', { containerDivId: 'installationContainer', elementDivId: 'installationItem' }),
 *       ]
 *     }
 *   }
 * };
 * ```
 * Example output:
 * ```html
 * <div id="installationContainer">
 *  <div id="installationItem">
 *      <h1>Markdown Title Content</h1>
 *  </div>
 * </div>
 * ```
 * @param {string} markdownFileSource source of the markdown file to be converted into HTML, then passed into the sectionBuilder
 * @param {SingleMarkdownSectionBuilderExtras} extras The `containerIdAttribute` will be attached as the `id` attribute value for the container div.  The `elementIdAttribute` will be attached as the `id` attribute value for the interior element div.  Passing `null` for either will omit the selector.
 * @returns A `MarkdownSectionBuilder` that will be used by the plugin to create the single markdown section.
 */
function createSingleMarkdownSection(markdownFileSource, extras) {
    return {
        sectionBuilder: function (markdownHtmls) {
            var containerIdAttribute = extras.containerDivId === null ? '' : " id=\"" + extras.containerDivId + "\"";
            var elementIdAttribute = extras.elementDivId === null ? '' : " id=\"" + extras.elementDivId + "\"";
            return "<div" + containerIdAttribute + "><div" + elementIdAttribute + ">" + markdownHtmls[0] + "</div></div>";
        },
        markdownFileSources: [markdownFileSource],
        extras: extras
    };
}

/**
 * Creates a section of a page from multiple markdown files, and adds the passed css selectors for inserting layout customization.
 *
 * Example markdown source files:
 * ```md
 * // ./sourceDir/file1.md
 * # First Markdown Title Content
 * ```
 * ```md
 * // ./sourceDir/file2.md
 * # Second Markdown Title Content
 * ```
 * Example usage:
 * ```typescript
 * export const config: ScullyConfig = {
 *   // ...
 *   routes: {
 *     '/path': {
 *       type: MultiMarkdownPageBuilderPlugin,
 *       sectionBuilders: [
 *         createMultipleMarkdownSection(['./sourceDir/file1.md', './sourceDir/file2.md'], { containerDivId: 'gridContainer', elementsDivClass: 'grid-item' })
 *       ]
 *     }
 *   }
 * };
 * ```
 * Example output:
 * ```html
 * <div id="gridContainer">
 *  <div id="element1" class="grid-item">
 *      <h1>First Markdown Title Content</h1>
 *  </div>
 *  <div id="element2" class="grid-item">
 *      <h1>Second Markdown Title Content</h1>
 *  </div>
 * </div>
 * ```
 * @param {string[]} markdownFileSources source of the markdown files to be converted into HTML, then passed into the sectionBuilder
 * @param {SingleMarkdownSectionBuilderExtras} extras The `containerIdAttribute` will be attached as the `id` attribute value for the container div.  The `elementsDivClass` will be attached as the `class` attribute value for the interior element div.  Passing `null` for either will omit the selector.
 * @returns A `MarkdownSectionBuilder` that will be used by the plugin to create the single markdown section.
 */
function createMultipleMarkdownSection(markdownFileSources, extras) {
    return {
        sectionBuilder: function (markdownHtmls) {
            var markdownFileIndex = 1;
            var combinedMarkdownHtml = markdownHtmls.reduce(function (combined, currentMarkdownHtml) {
                var idAttribute = " id=\"element" + markdownFileIndex++ + "\"";
                var classAttribute = extras.elementsDivClass === null ? '' : " class=\"" + extras.elementsDivClass + "\"";
                return combined + ("<div" + idAttribute + classAttribute + ">" + currentMarkdownHtml + "</div>");
            }, '');
            var idAttribute = extras.containerDivId === null ? '' : " id=\"" + extras.containerDivId + "\"";
            return "<div" + idAttribute + ">" + combinedMarkdownHtml + "</div>";
        },
        markdownFileSources: markdownFileSources,
        extras: extras
    };
}

/*
 * Public API Surface of scully-multi-markdown-page-builder-plugin
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MultiMarkdownPageBuilderPlugin, createMultipleMarkdownSection, createSingleMarkdownSection };
//# sourceMappingURL=scully-multi-markdown-page-builder-plugin.js.map
